@@ -1,6 +1,50 @@
-# $Id: Hooks.pm 443 2004-12-12 19:48:04Z sungo $
+# $Id: Hooks.pm 620 2005-12-10 22:49:11Z sungo $
 package POE::API::Hooks;
-# pod at bottom
+
+=pod
+
+=head1 NAME
+
+POE::API::Hooks - Implement lightweight hooks into POE
+
+=head1 SYNOPSIS
+
+  use POE;
+  use POE::API::Hooks;
+
+  POE::API::Hooks->add(
+      before_event_dispatch  => \&do_something,
+      after_event_dispatch   => \&do_something,
+      before_session_create  => \&do_something,
+      after_session_create   => \&do_something,
+      before_event_enqueue   => \&do_something,
+      after_event_enqueue    => \&do_something,
+  );
+
+  # ... carry on with life as normal ...
+
+=head1 DISCUSSION
+
+This module adds lightweight hooks into the inner workings of POE.
+Currently, one can add hooks into POE that get called before/after an
+event is dispatched, before/after a Session is created, and/or
+before/after an event is enqueued.  These callbacks receive the exact
+same argument list as their Kernel/Session counterpart. For event
+dispatch related callbacks, see C<_dispatch_event> and
+C<_data_ev_enqueue> in L<POE::Kernel>.
+For session related callbacks, see C<create> in L<POE::Session>.
+
+=head1 WARNING
+
+This interface is designed for advanced users. Wrapping any of the
+available POE internals is a potentially dangerous operation. Please
+read the POE documentation and source code carefully before attempting
+an override of this nature.
+
+=head1 METHODS
+
+=cut
+
 
 BEGIN {
 	use POE;
@@ -24,7 +68,7 @@ use strict;
 
 use Params::Validate qw(validate CODEREF);
 
-our $VERSION = '1.'.sprintf "%04d", (qw($Rev: 443 $))[1];
+our $VERSION = '2.01';
 
 my %HOOKS = (
 	before_event_dispatch  => [],
@@ -37,6 +81,36 @@ my %HOOKS = (
 	after_event_enqueue    => [],
 );
 
+
+
+=head2 add
+
+
+  POE::API::Hooks->add(
+      before_event_dispatch  => \&do_something,
+      after_event_dispatch   => \&do_something,
+
+      before_session_create  => \&do_something,
+      after_session_create   => \&do_something,
+
+      before_event_enqueue   => \&do_something,
+      after_event_enqueue    => \&do_something,
+  );
+
+C<add> is the only public method of this class. The method installs the
+requested hooks. The list of options above contains all possible hooks.
+No hook is required so the user may set only one hook if they desire.
+
+The callback provided for each hook will be called, obviously, when that
+state occurs. C<before_event_dispatch> will be called just before event
+dispatch, for instance. These callbacks are passed whatever arguments
+are provided to the wrapped subroutine. For instance, if a
+C<before_session_create> callback is registered and a user calls
+C<< POE::Session->create('POE') >>, the registered callback will receive
+C<('POE::Session', 'POE')> in C<@_>.
+
+=cut
+
 sub add {
 	my $class = shift;
 	validate(@_, {
@@ -95,6 +169,17 @@ sub add {
 	return;
 }
 
+
+
+=begin developers
+
+=head2 ima_bad_monkey_dispatch_event
+
+An override for POE's POE::Kernel::_dispatch_event which allows us to put 
+our ... hooks into POE's dispatch loop
+
+=cut
+
 sub ima_bad_monkey_dispatch_event {
 	my @args = @_;
 	if(@{ $HOOKS{before_event_dispatch} }) {
@@ -114,6 +199,17 @@ sub ima_bad_monkey_dispatch_event {
 	return $rc;
 }
 
+
+
+=head2 ima_bad_monkey_session_create
+
+An override for POE's POE::Session::create which allows us to put 
+our ... hooks into POE's session creation. Why don't we override 
+POE::Session::new you ask? Well, that's a deprecated api and you 
+should stop using it.
+
+=cut
+
 sub ima_bad_monkey_session_create {
 	my @args = @_;
 	if(@{ $HOOKS{before_session_create} }) {
@@ -133,6 +229,13 @@ sub ima_bad_monkey_session_create {
 	return $rc;
 }
 
+=head2 ima_bad_monkey_data_ev_enqueue
+
+An override for POE's POE::Kernel::_data_ev_enqueue which allows us to
+put our ... hooks into an event's entrance into the queue.
+
+=cut
+
 sub ima_bad_monkey_data_ev_enqueue {
 	my @args = @_;
 	if(@{ $HOOKS{before_event_enqueue} }) {
@@ -152,46 +255,19 @@ sub ima_bad_monkey_data_ev_enqueue {
 	return $rc;
 }
 
+=pod
+
+=end developers
+
+=cut
+
 1;
 
 __END__
 
-=pod
-
-=head1 NAME
-
-POE::API::Hooks - Implement lightweight hooks into POE
-
-=head1 SYNOPSIS
-
-  use POE;
-  use POE::API::Hooks;
-
-  POE::API::Hooks->add(
-      before_event_dispatch  => \&do_something,
-      after_event_dispatch   => \&do_something,
-      before_session_create  => \&do_something,
-      after_session_create   => \&do_something,
-      before_event_enqueue   => \&do_something,
-      after_event_enqueue    => \&do_something,
-  );
-
-  # ... carry on with life as normal ...
-
-=head1 DISCUSSION
-
-This module adds lightweight hooks into the inner workings of POE.
-Currently, one can add hooks into POE that get called before/after an
-event is dispatched, before/after a Session is created, and/or
-before/after an event is enqueued.  These callbacks receive the exact
-same argument list as their Kernel/Session counterpart. For event
-dispatch related callbacks, see C<_dispatch_event> and
-C<_data_ev_enqueue> in L<POE::Kernel>.
-For session related callbacks, see C<create> in L<POE::Session>.
-
 =head1 BUGS
 
-When $_[KERNEL]->call($_[SESSION], "event") is used, _event_dispatch is
+When C<< $_[KERNEL]->call($_[SESSION], "event") >> is used, _event_dispatch is
 bypassed.  This is an optimization, not a bug, but it will cause the
 event_dispatch hook to be bypassed under this condition.  Someone
 relying on the event_dispatch hook may call this a bug.
@@ -200,17 +276,9 @@ relying on the event_dispatch hook may call this a bug.
 
 Matt Cashner (sungo@pobox.com)
 
-=head1 DATE
-
-$Date: 2004-12-12 14:48:04 -0500 (Sun, 12 Dec 2004) $
-
-=head1 REVISION
-
-$Rev: 443 $
-
 =head1 LICENSE
 
-Copyright (c) 2004, Matt Cashner. All rights reserved.
+Copyright (c) 2004, 2008 Matt Cashner. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
@@ -222,7 +290,7 @@ met:
 
 =item * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 
-=item * Neither the name of the Matt Cashner nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
+=item * Neither the name of Matt Cashner nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 =back
 
